# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package FDN::Web::Array;

use strict;
use Exporter;
our @ISA = ( 'Exporter' );
our @EXPORT = qw (
	tableau_1d
	tableau_1d_text
	tableau_1d_tex
	tableau_2d
	tableau_2d_text
	tableau_2d_tex
	tableau_2cols
	tableau_2cols_text
	tableau_2cols_tex
	tableau_2d_pognon
	tableau_2d_pognon_text
	tableau_2d_pognon_tex
);
our @EXPORT_OK = ();
our %EXPORT_TAGS = ();

use Encode;
use FDN::Common::Config;
use FDN::Web::TemplateParse;

my $TPLDIR = '/usr/share/fdn/templates/array/';

sub tableau_1d($$$;$) {
	my ( $rewrite_file, $rewrite, $infos, $titre ) = @_;
	return __tableau_1d($rewrite_file, $rewrite, $infos, 'html', $titre);
}

sub tableau_1d_text($$$;$) {
	my ( $rewrite_file, $rewrite, $infos, $titre ) = @_;
	return __tableau_1d($rewrite_file, $rewrite, $infos, 'text', $titre);
}

sub tableau_1d_tex($$$;$) {
	my ( $rewrite_file, $rewrite, $infos, $titre ) = @_;
	return __tableau_1d($rewrite_file, $rewrite, $infos, 'tex', $titre);
}

sub __tableau_1d($$$$;$) {
	my ( $rewrite_file, $rewrite, $infos, $type, $titre ) = @_;
	my $t = Rewrite_Table($rewrite_file,$rewrite,$infos);
	if ( defined $titre ) {
		$t->{'titre'} = $titre;
	}
	my $res = remplace($TPLDIR.$type.'-tpl/arr1.tpl',$t);
	if ( exists $t->{'encoding'} && $t->{'encoding'} eq 'utf8' ) {
		Encode::_utf8_on($res);
	}
	return $res;
}

sub tableau_2cols($$) {
	my ( $infos, $titre ) = @_;
	return __tableau_2cols($infos, 'html', $titre);
}

sub tableau_2cols_text($$) {
	my ( $infos, $titre ) = @_;
	return __tableau_2cols($infos, 'text', $titre);
}

sub tableau_2cols_tex($$) {
	my ( $infos, $titre ) = @_;
	return __tableau_2cols($infos, 'tex', $titre);
}

sub __tableau_2cols($$$) {
	my ( $infos, $type, $titre ) = @_;
	my $hash = {};
	$hash->{titre}=$titre;
	$hash->{valeurs}=[];
	for my $k ( keys %$infos ) {
		my $v = {};
		$v->{nom} = $k;
		$v->{valeur} = $infos->{$k};
		push @{$hash->{valeurs}},$v;
	}
	return remplace($TPLDIR.$type.'-tpl/arr1.tpl',$hash,1);
}

sub __tableau_2d($$$$;$) {
	my ( $rewrite_file, $rewrite, $liste, $type, $titre ) = @_;
	my $tbl = { lignes => [] };
	my $t;
	for my $elt ( @$liste ) {
		$t = Rewrite_Table($rewrite_file,$rewrite,$elt);
		push @{$tbl->{lignes}},$t
	}
	$tbl->{titres}=Rewrite_Table($rewrite_file,$rewrite,{});
	if ( defined $titre ) {
		$tbl->{titre}=$titre;
	} else {
		$tbl->{titre}=$tbl->{titres}->{titre};
	}
	my $encoding = 'text';
	if ( exists $tbl->{titres}->{'encoding'} ) {
		$encoding = $tbl->{titres}->{'encoding'};
	}
	$tbl->{titres}=$tbl->{titres}->{valeurs};
	my $res = remplace($TPLDIR.$type.'-tpl/arr2.tpl',$tbl);
	if ( $encoding eq 'utf8' ) {
		Encode::_utf8_on($res);
	}
	return $res;
}

sub tableau_2d($$$;$) {
	my ( $rewrite_file, $rewrite, $liste, $titre ) = @_;
	return __tableau_2d($rewrite_file, $rewrite, $liste, 'html', $titre);
}

sub tableau_2d_text($$$;$) {
	my ( $rewrite_file, $rewrite, $liste, $titre ) = @_;
	return __tableau_2d($rewrite_file, $rewrite, $liste, 'text', $titre);
}

sub tableau_2d_tex($$$;$) {
	my ( $rewrite_file, $rewrite, $liste, $titre ) = @_;
	return __tableau_2d($rewrite_file, $rewrite, $liste, 'tex', $titre);
}

sub __tableau_2d_pognon($$$$$;$) {
	my ( $rewrite_file, $rewrite, $label, $liste, $type, $titre ) = @_;
	my $page = {};
	$page->{'TABLE'} = __tableau_2d($rewrite_file,$rewrite,$liste,$type,$titre);
	$page->{'totaux'} = [];
	my $ht	= { 'nom' => 'Total HT', 'total' => 0 };
	my $tva = { 'nom' => 'Total TVA', 'total' => 0 };
	my $ttc = { 'nom' => 'Total TTC',  'total' => 0 };
	for my $elt ( @$liste ) {
		$ht->{'total'}	+= $elt->{$label.'_HT'};
		$ttc->{'total'} += $elt->{$label.'_TTC'};
	}
	$tva->{'total'} = $ttc->{'total'} - $ht->{'total'};
	$ht->{'total'}	= prt_euro($ht->{'total'});
	$tva->{'total'} = prt_euro($tva->{'total'});
	$ttc->{'total'} = prt_euro($ttc->{'total'});
	push @{$page->{'totaux'}}, $ht;
	push @{$page->{'totaux'}}, $tva;
	push @{$page->{'totaux'}}, $ttc;
	return remplace($TPLDIR.$type.'-tpl/pognon.tpl',$page);
}

sub tableau_2d_pognon($$$$;$) {
	my ( $rewrite_file, $rewrite, $label, $liste, $titre ) = @_;
	return __tableau_2d_pognon($rewrite_file,$rewrite,$label,$liste,'html',$titre);
}

sub tableau_2d_pognon_text($$$$;$) {
	my ( $rewrite_file, $rewrite, $label, $liste, $titre ) = @_;
	return __tableau_2d_pognon($rewrite_file,$rewrite,$label,$liste,'text',$titre);
}

sub tableau_2d_pognon_tex($$$$;$) {
	my ( $rewrite_file, $rewrite, $label, $liste, $titre ) = @_;
	return __tableau_2d_pognon($rewrite_file,$rewrite,$label,$liste,'tex',$titre);
}

sub prt_euro($) {
	my ($ct)=@_;
	$ct =~ s/(\d{2})$/,$1/;
	return $ct;
}

1;
